
/**
 * 
 * TurmaList = Lista(FichaAluno)
 * 
 * Esta  uma m implementao de Turma porque tendo cada aluno
 * um cdigo unvoco Turma deveria ser implementada com sendo
 * uma Tabela de Hashing de Nmero -> FichaAluno, ou seja, uma
 * correspondncia unvoca entre Nmero de Aluno e a sua Ficha.
 * 
 * Porm, o objectivo  estudar e usar o tipo ArrayList<String> 
 * e o novo for() apresentado em JAVA5.
 * 
 * @author F. Mrio Martins
 * @version 03/05/2005
 */
import java.util.*;
public class TurmaList {

   // Variveis de Instncia
    private ArrayList<FichaAluno> turma; 
    
    // Construtores
		
	/** Construtor por omisso */	
	public TurmaList() { turma = new ArrayList<FichaAluno>(); }	
	
	/** Construtor por cpia */
	public TurmaList(TurmaList t) {
	    turma = new ArrayList<FichaAluno>();
	    for(FichaAluno fa : t.daFichas()) turma.add(fa.clone());
	}
	
    /** Insere uma coleco de Fichas em turma */
	public TurmaList(Collection<FichaAluno> colfichas) { 
	    turma = new ArrayList<FichaAluno>();
	    for(FichaAluno ficha : colfichas) turma.add(ficha.clone());
	    // turma.addAll(colfichas); => partilha de endereos
	}
	
	// Mtodos de Instncia
	
	/** Determina o nmero de alunos da Turma */	
	public int numAlunos() { return turma.size(); }
	
	/** Devolve uma cpia do ArrayList de FichaAluno */
	public ArrayList<FichaAluno> daFichas() {
	    ArrayList<FichaAluno> fichas = new ArrayList<FichaAluno>();
	    for(FichaAluno fa : turma) fichas.add(fa.clone());
	    return fichas;
	}
	
	/** Cria um conjunto com os nmeros de todos os alunos da turma */	
	public TreeSet<String> codigos() {
	    TreeSet<String> codigos = new TreeSet<String>();
	    for(FichaAluno ficha : turma) codigos.add(ficha.getNumero());
	    return codigos;
	 }

   /** Insere um novo aluno na turma */
    public void insereAluno(FichaAluno ficha) { turma.add(ficha.clone()); }

	/** Determina a  maiorMdia da turma */	
	public double maiorNotaTurma() {	    
	  double maiorNota = -1.0;	
      double mediaAluno;
	  for(FichaAluno ficha : turma) {
	       mediaAluno = ficha.getMedia();   
	       if(mediaAluno > maiorNota) 
	              maiorNota = mediaAluno;
	  }	 
	  return maiorNota;
    }
		   	
     /**
	 *  Devolve o conjunto dos cdigos dos alunos com nota superior  dada
	 *  como parmetro (ex cdigos dos alunos com nota > 12).
	 */
	 public HashSet<String> alunosMediaSuperior(double notaRef) {	    
	   HashSet<String> codigos = new HashSet<String>();  	
	   for(FichaAluno ficha : turma) 
	      if( ficha.getMedia() > notaRef ) codigos.add(ficha.getNumero());	          
	   return codigos;
    }
  
   /**
	* Verifica se um dado aluno cujo cdigo  dado est registado.
	* Tem que ser usado um Iterator porque o novo for() no permite
	* incluir condies de paragem da iterao.
	*/	
	public boolean existeAluno(String numAluno) {	    
	  Iterator<FichaAluno> it = turma.iterator();
	  FichaAluno ficha; String numero;
	  boolean existe = false;	  
	  while(it.hasNext() && !existe) {
	      numero = it.next().getNumero();
	      if(numero.equals(numAluno)) existe = true; 
	  }
	  return existe;
    }
    
    /** toString() */
    public String toString() {
       StringBuilder s = new StringBuilder();
       s.append("-----------  TURMA --------------------\n");
       for(FichaAluno ficha : turma) s.append(ficha.toString());   
	   return s.toString();
    }   
    
    /** clone */
    public TurmaList clone() { return new TurmaList(this); }
  
}	  
	    


